const CYBERSEARCH_Wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator).getMostRecentWindow("navigator:browser");

/*
Updates the color palette field with the HEX color
*/
function CYBERSEARCH_background_advanced(color){
	if(color.match(new RegExp("^#"))){
		document.getElementById('cybersearch_colorpicker_backgroundcolor').color = color;
	}
	else{
		document.getElementById('cybersearch_colorpicker_backgroundcolor').color = "#" + color;
	}
}


/*
Enable or disable fields depending on whether the user wants a custom background
*/
function CYBERSEARCH_background_toggle(is_disabled){
	if(is_disabled){
		document.getElementById('cybersearch_colorpicker_backgroundcolor').color	= "transparent";
		document.getElementById('cybersearch_colorpicker_backgroundcolor').disabled	= true;
		document.getElementById('cybersearch_textbox_backgroundcolor').value		= "";
		document.getElementById('cybersearch_textbox_backgroundcolor').disabled		= true;
	}
	else{
		document.getElementById('cybersearch_colorpicker_backgroundcolor').color	= "#EEEEEE";
		document.getElementById('cybersearch_colorpicker_backgroundcolor').disabled	= false;
		document.getElementById('cybersearch_textbox_backgroundcolor').value		= "#EEEEEE";
		document.getElementById('cybersearch_textbox_backgroundcolor').disabled		= false;
	}
}


/*
Writes the custom CSS file to the user's chrome directory
*/
function CYBERSEARCH_background_write_css(){
	var css_file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("UChrm", Components.interfaces.nsIFile);

	//Figure out what OS the user is running so that we can get the slashes correct
	//"WINNT" = Windows, "Linux" = Linux, "Darwin" = Mac OS X
	var computerOS = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULRuntime).OS;
	if("WINNT" == computerOS){
		css_file = css_file.path + "\\" + "cybersearchBackground.css";
	}
	else {
		css_file = css_file.path + "/" + "cybersearchBackground.css";
	}


	//See if the file already exists, and create it if it doesn't
	try{
		var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
		file.initWithPath(css_file);
		if(file.exists() == false){
			file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420);
		}
	}
	catch(e){
		CYBERSEARCH_Wm.CYBERSEARCH_log_message("Error creating CyberSearch CSS file.");
		return;
	}

	//Prep the file for writing
	try{
		var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
		stream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);
	}
	catch(e){
		CYBERSEARCH_Wm.CYBERSEARCH_log_message("Error prepping CyberSearch CSS file.");
		return;
	}

	//Generate the content for the file
	var new_color	= document.getElementById('cybersearch_colorpicker_backgroundcolor').color;
	var search_result_icon = "list-style-image: url('chrome://cybersearch/content/"+ CYBERSEARCH_Wm.CYBERSEARCH_defaultEngine + ".png');";
	var content		= "richlistitem[type ~= 'cybersearch'] > hbox {background-color:" + new_color + ";" + search_result_icon + "}";


	//Write the CSS to the file
	try{
		stream.write(content, content.length);
		stream.close();
	}
	catch(e){
		CYBERSEARCH_Wm.CYBERSEARCH_log_message("Error writing CyberSearch CSS file.");
		return;
	}

	//Reload the stylesheet so that the changes are seen without restarting.
	CYBERSEARCH_Wm.CYBERSEARCH_load_custom_css();
}


/*
Hide the address field if it's not needed
*/
function CYBERSEARCH_hide_address_field(hide_me){
	var field_address = document.getElementById('cybersearch_row_keyword_address');

	if(hide_me){
		field_address.setAttribute("hidden", "true");
	}
	else{
		field_address.setAttribute("hidden", "false");
	}
}


/*
Update the icon preview to include the site's favicon
*/
function CYBERSEARCH_icon_update_preview(){
	//If the user hasn't entered in any URL's for filtering don't show the auto generate button
	var auto_icon		= document.getElementById('cybersearch_description_icon_generator');
	var address_field	= document.getElementById('cybersearch_textbox_keyword_address');
	if(address_field.value != "" && address_field.value != "example.com"){
		auto_icon.setAttribute("hidden", "false");
	}
	else{
		auto_icon.setAttribute("hidden", "true");
	}

	//Show the icon preview if there is an icon URL specified
	var icon_preview	= document.getElementById('cybersearch_image_keyword_icon');
	var icon_field		= document.getElementById('cybersearch_textbox_keyword_icon');
	if(icon_field.value != ""){
		icon_preview.setAttribute("src", icon_field.value);
	}
	else{
		icon_preview.setAttribute("src", "chrome://cybersearch/content/questionmark.png");
	}
}


/*
Updates the icon URL field with an auto-generated URL
*/
function CYBERSEARCH_icon_update_url(overwrite_url){
	var new_icon_url		= document.getElementById('cybersearch_textbox_keyword_address').value.split(",")[0];
	var current_icon_url	= document.getElementById('cybersearch_textbox_keyword_icon').value;

	//See if an icon URL can be auto generated
	var regex = new RegExp(/([0-9a-zA-Z-]+\.)?[0-9a-zA-Z-]+?\.[0-9a-zA-Z-]{1,6}/i);
	var regex_result = "";
	if(regex_result = regex.exec(new_icon_url)){
		new_icon_url = regex_result[0];
	}
	else{
		CYBERSEARCH_icon_update_preview();
		return; //Can't generate icon URL
	}

	//Format the favicon URL properly
	new_icon_url = "http://s2.googleusercontent.com/s2/favicons?domain=" + new_icon_url;

	//Replace the icon if the field is blank, or if the user clicked the "Auto" button
	if(current_icon_url == "" || overwrite_url){
		document.getElementById('cybersearch_textbox_keyword_icon').value = new_icon_url;
	}

	CYBERSEARCH_icon_update_preview();
}


/*
Imports keywords used by Firefox
*/
function CYBERSEARCH_import_firefox_keywords(){
	//Imports icons from the browser search engine manager
	nsISearchService = Components.classes['@mozilla.org/browser/search-service;1'].getService(Components.interfaces.nsIBrowserSearchService);
	firefox_engines = nsISearchService.getEngines({});

	var count_imported	= 0;
	var count_dupes		= 0;
	var is_duplicate	= 0;
	for(var i=0; i < firefox_engines.length; i++){
		if(firefox_engines[i].alias != null){
			var new_keyword = {};
			new_keyword.address	= firefox_engines[i].searchForm.match(new RegExp(/([0-9a-zA-Z-]+\.)?[0-9a-zA-Z-]+?\.[0-9a-zA-Z-]{1,6}/i))[0];
			new_keyword.icon	= "";
			new_keyword.type	= "web";
			new_keyword.trigger	= firefox_engines[i].alias;

			if(new_keyword.address != ""){
				//See if keyword already exists
				is_duplicate = CYBERSEARCH_is_dupe_keyword(new_keyword);

				if(is_duplicate < 0){
					//Push the keyword onto the array
					CYBERSEARCH_Wm.CYBERSEARCH_keywords.push(new_keyword);
					count_imported++;
				}
				else{
					//It's a dupe... chalk it up for the love letter
					count_dupes++;
				}
			}
		}
	}

	//Save the keywords, regardless of whether any are actually new
	var save_json = JSON.stringify(CYBERSEARCH_Wm.CYBERSEARCH_keywords);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setCharPref("extensions.cybersearch.keywords", save_json);

	CYBERSEARCH_keyword_refresh_menu();

	//Alert the user with a status update
	if(count_dupes > 0 && count_imported > 0){
		//Duplicate keywords were found... notify the user
		alert("So close. There's " + count_dupes + " that I ignored because of duplicate keywords. The rest have been added to the end of the keyword list.");
	}
	else if(count_dupes > 0 && 0 == count_imported){
		//Duplicate keywords were found... but no other keywords were available
		alert("Move along, nothing to see here. Nothing could be imported, and there was " + count_dupes + " that I ignored because of duplicate keywords.");
	}
	else if(0 == count_imported){
		//No keywords were imported
		alert("Well then. That was easy. There were no valid keywords for me to import.");
	}
	else{
		//Smooth sailin'
		alert("Cool beans. The search engines that you assigned keywords to have been added to the end of the keyword list.");
	}

}


/*
Check to see if a keyword is already being used, if so return the index number
*/
function CYBERSEARCH_is_dupe_keyword(new_keyword){
	for(var i = 0; i < CYBERSEARCH_Wm.CYBERSEARCH_keywords.length; i++){
		if(CYBERSEARCH_Wm.CYBERSEARCH_keywords[i].trigger == new_keyword.trigger){
			//This keyword is a duplicate
			return i;
		}
	}

	//Keyword is not a duplicate
	return -1;
}


/*
Creates a menu item based on the attributes of a keyword
*/
function CYBERSEARCH_keyword_create_menu_item(keyword){
	var menu_element = document.createElement("menuitem");
	menu_element.setAttribute("label", keyword.trigger);

	if (typeof(keyword.type) == "undefined") keyword.type = "";

	//Truncate long descriptions/URL's
	var pretty_type			= keyword.type.substr(0, 1).toUpperCase() + keyword.type.substr(1);
	var pretty_engine		= keyword.engine.substr(0, 1).toUpperCase() + keyword.engine.substr(1);
	var menu_description	= "[" + pretty_engine + " " + pretty_type + "] " + keyword.address;
	if(menu_description.length > 100){
		menu_description = menu_description.substr(0,100) + "...";
	}
	menu_element.setAttribute("description", menu_description);

	//Generate action command
	menu_element.setAttribute("oncommand", "CYBERSEARCH_keyword_update_fields();");
	menu_element.setAttribute("value", JSON.stringify(keyword));

	return menu_element;
}


/*
Refresh the contents of the keyword menu for the initial loading or to reflect any changes that have been made
*/
function CYBERSEARCH_keyword_refresh_menu(){
	//Remove existing items in the list
	var keyword_menu = document.getElementById('cybersearch_menupopup_keywords');
	if(keyword_menu.hasChildNodes()){
		while(keyword_menu.childNodes.length > 0){
			keyword_menu.removeChild(keyword_menu.firstChild);
		}
	}

	//When creating a new keyword grab the URL from the browser window
	var browser_address = "";
	var regex			= new RegExp(/([0-9a-zA-Z-]+\.)?[0-9a-zA-Z-]+?\.[0-9a-zA-Z-]{1,6}/i);
	var regex_result	= "";
	if(regex_result = regex.exec(CYBERSEARCH_Wm.gURLBar.value)){
		browser_address = regex_result[0];
	}

	//Populate the menu again
	keyword_menu.appendChild(CYBERSEARCH_keyword_create_menu_item({"address":browser_address, "engine":"google", "icon":"", "type":"web", "trigger":"New Keyword"}));
	for(var i=0; i < CYBERSEARCH_Wm.CYBERSEARCH_keywords.length; i++){
		keyword_menu.appendChild(CYBERSEARCH_keyword_create_menu_item(CYBERSEARCH_Wm.CYBERSEARCH_keywords[i]));
	}

	//Select the first item of the list
	document.getElementById('cybersearch_menulist_keywords').selectedIndex = 0;
	CYBERSEARCH_keyword_update_fields();
}

/*
Saves only the keywords to your settings (canceling the preferences window will not undo this action)
*/
function CYBERSEARCH_keyword_save(save_action, fail_quietly){
	var selected_id = document.getElementById('cybersearch_menulist_keywords').selectedIndex;

	//Keyword object initialization used to insert into array later on, if needed
	var new_keyword = {};
	new_keyword.address	= document.getElementById('cybersearch_textbox_keyword_address').value;
	new_keyword.icon	= document.getElementById('cybersearch_textbox_keyword_icon').value;
	new_keyword.type	= document.getElementById('cybersearch_menulist_keyword_type').selectedItem.value;
	new_keyword.trigger	= document.getElementById('cybersearch_textbox_keyword_trigger').value;

	//Determine the search engine
	new_keyword.engine	= "google";
	new_keyword.type	= "web";

	//Make sure keyword icon URL has the http:// in front of it, if an icon is being used
	if(!new_keyword.icon.match("^http://") && new_keyword.icon != ""){
		new_keyword.icon = "http://" + new_keyword.icon;
	}

	//Can't leave the keyword field blank... for obvious reasons
	if(new_keyword.trigger == "" && "add" == save_action){
		if(!fail_quietly){
			alert("You cannot leave the keyword field blank.");
		}
		return;
	}

	//Check for duplicate keywords before saving; drop out if one is found
	if("add" == save_action){
		var is_duplicate = CYBERSEARCH_is_dupe_keyword(new_keyword);
		if(is_duplicate > -1 && selected_id != (is_duplicate + 1)){
			//Yes, it is really a duplicate
			if(!fail_quietly){
				alert("Um, yeah. So this keyword has already been used. Make me happy and choose a different one.");
			}
			return;
		}
	}

	//Website URL was specified on incompatible type
	if(new_keyword.type != "web" && new_keyword.type != "Web"){
		new_keyword.address = "";
	}

	if(0 == selected_id){
		//It's a new keyword (not an edit)
		if("delete" == save_action){
			return;
		}
		else{
			CYBERSEARCH_Wm.CYBERSEARCH_keywords.push(new_keyword);
		}
	}
	else{
		//Update or delete the keyword
		if("delete" == save_action){
			CYBERSEARCH_Wm.CYBERSEARCH_keywords.splice(selected_id - 1, 1);
		}
		else{
			CYBERSEARCH_Wm.CYBERSEARCH_keywords[selected_id - 1] = new_keyword;
		}
	}

	//Save the keywords
	var save_json = JSON.stringify(CYBERSEARCH_Wm.CYBERSEARCH_keywords);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setCharPref("extensions.cybersearch.keywords", save_json);

	//Refresh the keyword menu to reflect the changes
	CYBERSEARCH_keyword_refresh_menu();
}


/*
Sorts keywords in ascending or descending order
*/
function CYBERSEARCH_keyword_sort(sort_ascending){
	if(sort_ascending){
		var confirm_sort = confirm("Press OK to sort the list by keyword in ascending (A-Z) order.");
	}
	else{
		var confirm_sort = confirm("Press OK to sort the list by keyword in descending (Z-A) order.");
	}

	if(confirm_sort){
		//Just use a bubblesort because I'm lazy, and it works
		var keywords = CYBERSEARCH_Wm.CYBERSEARCH_keywords;
		for(var i = 0; i < (keywords.length - 1); i++){
			for(var j = i+1; j < keywords.length; j++){
				var swap_them = false;
				if(sort_ascending){
					//Check for swap (ascending)
					if(keywords[j].trigger < keywords[i].trigger){
						swap_them = true;
					}
				}
				else{
					//Check for swap (descending)
					if(keywords[j].trigger > keywords[i].trigger){
						swap_them = true;
					}
				}

				if(swap_them){
					//The heart of a bubblesort... swapping the two values
					var dummy_keyword	= keywords[i];
					keywords[i]			= keywords[j];
					keywords[j]			= dummy_keyword;
				}
			}
		}

		//Save the keywords
		var save_json = JSON.stringify(CYBERSEARCH_Wm.CYBERSEARCH_keywords);
		CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setCharPref("extensions.cybersearch.keywords", save_json);

		CYBERSEARCH_keyword_refresh_menu();
	}

	document.getElementById('cybersearch_menulist_keywords_sort').selectedIndex = 0;
}


/*
Updates neccessary fields when switching to a different keyword
*/
function CYBERSEARCH_keyword_update_fields(){
	var keyword_selected	= document.getElementById('cybersearch_menulist_keywords').selectedItem;
	var keyword_index		= document.getElementById('cybersearch_menulist_keywords').selectedIndex;
	var keyword				= JSON.parse(keyword_selected.value);

	//Find the ID number for this specific keyword type
	var keyword_types	= new Array('web');
	var keyword_type_id = keyword_types.indexOf(keyword.type);

	//Make the "New Keyword" not actually show that in the trigger box
	if(0 == keyword_index){
		keyword.trigger = "";
	}

	//Update the respective fields
	document.getElementById('cybersearch_textbox_keyword_trigger').value		= keyword.trigger;
	document.getElementById('cybersearch_textbox_keyword_address').value		= keyword.address;
	document.getElementById('cybersearch_textbox_keyword_icon').value			= keyword.icon;
	document.getElementById('cybersearch_menulist_keyword_type').selectedIndex	= keyword_type_id;

	//Should the website address field be displayed? (yes if web, blogs, or images)
	if("web" == keyword.type || "Web" == keyword.type){
		CYBERSEARCH_hide_address_field(false);
	}
	else{
		CYBERSEARCH_hide_address_field(true);
	}

	CYBERSEARCH_icon_update_preview();
}


/*
Initializes all fields with their respective values
*/
function CYBERSEARCH_options_initialize(){
	//Reinitialize all variable values before starting
	CYBERSEARCH_Wm.CYBERSEARCH_initialize_variables();

	//Maximum number of results to be displayed in the address bar
	document.getElementById('cybersearch_menulist_maxResultsList').selectedIndex	= parseInt(CYBERSEARCH_Wm.CYBERSEARCH_maxRichResults / 5) - 1;

	//Custom background color
	document.getElementById('cybersearch_colorpicker_backgroundcolor').color		= CYBERSEARCH_Wm.CYBERSEARCH_backgroundColor;
	if("transparent" == CYBERSEARCH_Wm.CYBERSEARCH_backgroundColor){
		document.getElementById('cybersearch_checkbox_noBackground').checked		= true;
		document.getElementById('cybersearch_colorpicker_backgroundcolor').disabled	= true;
		document.getElementById('cybersearch_textbox_backgroundcolor').value		= "";
		document.getElementById('cybersearch_textbox_backgroundcolor').disabled		= true;
	}
	else{
		document.getElementById('cybersearch_checkbox_noBackground').checked		= false;
		document.getElementById('cybersearch_textbox_backgroundcolor').value		= CYBERSEARCH_Wm.CYBERSEARCH_backgroundColor;
	}

	//Display descriptions when selecting options with the keyboard
	var engine_list		= new Array('google');
	var engine_index	= engine_list.indexOf(CYBERSEARCH_Wm.CYBERSEARCH_defaultEngine);
	document.getElementById('cybersearch_menulist_defaultEngine').selectedIndex		= engine_index;

	//Display descriptions when selecting options with the keyboard
	document.getElementById('cybersearch_checkbox_displayDescriptions').checked		= CYBERSEARCH_Wm.CYBERSEARCH_displayDescriptions;

	//Display CyberSearch Options in Tools Menu
	document.getElementById('cybersearch_checkbox_displayOptions').checked			= CYBERSEARCH_Wm.CYBERSEARCH_displayOptions;

	//Display Add to CyberSearch in Context Menu
	document.getElementById('cybersearch_checkbox_displayOptionsContext').checked	= CYBERSEARCH_Wm.CYBERSEARCH_displayOptionsContext;

	//Display Snapback option in address bar
	document.getElementById('cybersearch_checkbox_displaySnapback').checked			= CYBERSEARCH_Wm.CYBERSEARCH_displaySnapback;

	//Auto search domains
	document.getElementById('cybersearch_checkbox_domainSearch').checked			= CYBERSEARCH_Wm.CYBERSEARCH_domainSearch;

	//Show only search results when a keyword is recognized
	document.getElementById('cybersearch_checkbox_enterGoToGoogle').checked			= CYBERSEARCH_Wm.CYBERSEARCH_enterGoToGoogle;

	//Show only search results when a keyword is recognized
	document.getElementById('cybersearch_checkbox_hideHistoryOnKeyword').checked	= CYBERSEARCH_Wm.CYBERSEARCH_hideHistoryOnKeyword;

	//Show only search results when a keyword is recognized
	document.getElementById('cybersearch_checkbox_hideSearchNoKeyword').checked		= CYBERSEARCH_Wm.CYBERSEARCH_hideSearchNoKeyword;

	//Prefix required before keywords
	document.getElementById('cybersearch_textbox_keywordPrefix').value				= CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefix;

	//Require prefix before keywords (already required before domain searches)
	document.getElementById('cybersearch_checkbox_keywordPrefixOnAll').checked		= CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefixOnAll;

	//Keywords are enabled?
	document.getElementById('cybersearch_checkbox_keywordsEnabled').checked			= CYBERSEARCH_Wm.CYBERSEARCH_keywordsEnabled;

	//Maximum number of rows to be displayed in the address bar
	document.getElementById('cybersearch_menulist_maxRows').selectedIndex			= CYBERSEARCH_Wm.CYBERSEARCH_maxRows - 1;

	//The page keyword
	document.getElementById('cybersearch_textbox_pageKeyword').value				= CYBERSEARCH_Wm.CYBERSEARCH_pageKeyword;

	//Enable the multi-page keyword
	document.getElementById('cybersearch_checkbox_pageKeywordEnabled').checked		= CYBERSEARCH_Wm.CYBERSEARCH_pageKeywordEnabled;


	//Show thumbnails
	document.getElementById('cybersearch_checkbox_showThumbnails').checked			= CYBERSEARCH_Wm.CYBERSEARCH_showThumbnails;

	CYBERSEARCH_keyword_refresh_menu();
}


/*
Resets all settings back to their default values
*/
function CYBERSEARCH_options_restore_defaults(){
	var preferences = new Array('browser.urlbar.maxRichResults',
								'extensions.cybersearch.backgroundColor',
								'extensions.cybersearch.defaultEngine',
								'extensions.cybersearch.displayDescriptions',
								'extensions.cybersearch.displayOptions',
								'extensions.cybersearch.displayOptionsContext',
								'extensions.cybersearch.displaySnapback',
								'extensions.cybersearch.domainSearch',
								'extensions.cybersearch.enterGoToGoogle',
								'extensions.cybersearch.hideHistoryOnKeyword',
								'extensions.cybersearch.hideSearchNoKeyword',
								'extensions.cybersearch.keywordPrefix',
								'extensions.cybersearch.keywordPrefixOnAll',
								'extensions.cybersearch.keywords',
								'extensions.cybersearch.keywordsEnabled',
								'extensions.cybersearch.maxRows',
								'extensions.cybersearch.pageKeyword',
								'extensions.cybersearch.pageKeywordEnabled');

	for(var i=0; i < preferences.length; i++){
		try{
			CYBERSEARCH_Wm.CYBERSEARCH_PREFS.clearUserPref(preferences[i]);
		}
		catch(e){
			CYBERSEARCH_Wm.CYBERSEARCH_log_message("CyberSearch error resetting preference " + preferences[i] + ".");
		}
	}

	//Background color reset needs a little extra TLC, but I'm too lazy to delete the file.
	document.getElementById('cybersearch_colorpicker_backgroundcolor').color = "transparent";
	CYBERSEARCH_background_write_css();

	//Reset the interface
	CYBERSEARCH_options_initialize();
}


/*
Save all of the settings
*/
function CYBERSEARCH_options_save(){
	//Grab settings
	CYBERSEARCH_Wm.CYBERSEARCH_maxResultsList			= (document.getElementById('cybersearch_menulist_maxResultsList').selectedIndex + 1) * 5;
	CYBERSEARCH_Wm.CYBERSEARCH_defaultEngine			= document.getElementById('cybersearch_menulist_defaultEngine').selectedItem.value;
	CYBERSEARCH_Wm.CYBERSEARCH_displayDescriptions		= document.getElementById('cybersearch_checkbox_displayDescriptions').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_displayOptions			= document.getElementById('cybersearch_checkbox_displayOptions').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_displayOptionsContext	= document.getElementById('cybersearch_checkbox_displayOptionsContext').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_displaySnapback			= document.getElementById('cybersearch_checkbox_displaySnapback').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_domainSearch				= document.getElementById('cybersearch_checkbox_domainSearch').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_enterGoToGoogle			= document.getElementById('cybersearch_checkbox_enterGoToGoogle').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_hideHistoryOnKeyword		= document.getElementById('cybersearch_checkbox_hideHistoryOnKeyword').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_hideSearchNoKeyword		= document.getElementById('cybersearch_checkbox_hideSearchNoKeyword').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefix			= document.getElementById('cybersearch_textbox_keywordPrefix').value;
	CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefixOnAll		= document.getElementById('cybersearch_checkbox_keywordPrefixOnAll').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_keywordsEnabled			= document.getElementById('cybersearch_checkbox_keywordsEnabled').checked;
	CYBERSEARCH_Wm.CYBERSEARCH_maxRows					= document.getElementById('cybersearch_menulist_maxRows').selectedIndex + 1;
	CYBERSEARCH_Wm.CYBERSEARCH_pageKeyword				= document.getElementById('cybersearch_textbox_pageKeyword').value;
	CYBERSEARCH_Wm.CYBERSEARCH_pageKeywordEnabled		= document.getElementById('cybersearch_checkbox_pageKeywordEnabled').checked;

	CYBERSEARCH_Wm.CYBERSEARCH_showThumbnails			= document.getElementById('cybersearch_checkbox_showThumbnails').checked;


	//Store settings
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setIntPref("browser.urlbar.maxRichResults",					CYBERSEARCH_Wm.CYBERSEARCH_maxResultsList);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setCharPref("extensions.cybersearch.defaultEngine",			CYBERSEARCH_Wm.CYBERSEARCH_defaultEngine);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.displayDescriptions",		CYBERSEARCH_Wm.CYBERSEARCH_displayDescriptions);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.displayOptions",			CYBERSEARCH_Wm.CYBERSEARCH_displayOptions);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.displayOptionsContext",	CYBERSEARCH_Wm.CYBERSEARCH_displayOptionsContext);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.displaySnapback",			CYBERSEARCH_Wm.CYBERSEARCH_displaySnapback);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.domainSearch",				CYBERSEARCH_Wm.CYBERSEARCH_domainSearch);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.enterGoToGoogle",			CYBERSEARCH_Wm.CYBERSEARCH_enterGoToGoogle);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.hideHistoryOnKeyword",		CYBERSEARCH_Wm.CYBERSEARCH_hideHistoryOnKeyword);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.hideSearchNoKeyword",		CYBERSEARCH_Wm.CYBERSEARCH_hideSearchNoKeyword);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setCharPref("extensions.cybersearch.keywordPrefix",			CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefix);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.keywordPrefixOnAll",		CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefixOnAll);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.keywordsEnabled",			CYBERSEARCH_Wm.CYBERSEARCH_keywordsEnabled);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setIntPref("extensions.cybersearch.maxRows",					CYBERSEARCH_Wm.CYBERSEARCH_maxRows);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setCharPref("extensions.cybersearch.pageKeyword",				CYBERSEARCH_Wm.CYBERSEARCH_pageKeyword);
	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.pageKeywordEnabled",		CYBERSEARCH_Wm.CYBERSEARCH_pageKeywordEnabled);

	CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setBoolPref("extensions.cybersearch.showThumbnails",			CYBERSEARCH_Wm.CYBERSEARCH_showThumbnails);

	//Display CyberSearch options in the Tools menu if enabled
	if(CYBERSEARCH_Wm.CYBERSEARCH_displayOptions)			CYBERSEARCH_Wm.document.getElementById('cybersearch_options').setAttribute('hidden', 'false');
	else													CYBERSEARCH_Wm.document.getElementById('cybersearch_options').setAttribute('hidden', 'true');

	//Display Add to CyberSearch option in the Context Menu if enabled
	if(CYBERSEARCH_Wm.CYBERSEARCH_displayOptionsContext)	CYBERSEARCH_Wm.document.getElementById('cybersearch_options_context').setAttribute('hidden', 'false');
	else													CYBERSEARCH_Wm.document.getElementById('cybersearch_options_context').setAttribute('hidden', 'true');

	//Display CyberSearch Snapback option in the address bar if enabled
	if(CYBERSEARCH_Wm.CYBERSEARCH_displaySnapback)			CYBERSEARCH_Wm.document.getElementById('cybersearch_snapback').setAttribute('hidden', 'false');
	else													CYBERSEARCH_Wm.document.getElementById('cybersearch_snapback').setAttribute('hidden', 'true');

	//Save the background color only if it has changed (prevents writing to file every time)
	var new_color = document.getElementById('cybersearch_colorpicker_backgroundcolor').color;
	if(CYBERSEARCH_Wm.CYBERSEARCH_backgroundColor != new_color){
		CYBERSEARCH_Wm.CYBERSEARCH_backgroundColor = new_color;
		CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setCharPref("extensions.cybersearch.backgroundColor", CYBERSEARCH_Wm.CYBERSEARCH_backgroundColor);
		//CYBERSEARCH_background_write_css();
	}
	CYBERSEARCH_background_write_css();

	//Update the keyword if the user was editing one
	CYBERSEARCH_keyword_save('add', true);
}


/*
Export CyberSearch settings to a text file
*/
function CYBERSEARCH_settings_export(){
	const nsIFilePicker = Components.interfaces.nsIFilePicker;
	var filepicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

	//Display the file picker for the user
	filepicker.init(window, "File Location", nsIFilePicker.modeSave);
	filepicker.appendFilters(nsIFilePicker.filterText);
	filepicker.defaultExtension = "txt";
	filepicker.defaultString = "cybersearch_settings.txt";
	var filepicker_code = filepicker.show();

	//Grab the path they chose
	if(filepicker_code == nsIFilePicker.returnOK || filepicker_code == nsIFilePicker.returnReplace){
		var export_path = filepicker.file.path;

		//See if the file already exists, and create it if it doesn't
		try{
			var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			file.initWithPath(export_path);
			if(file.exists() == false){
				file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420);
			}
		}
		catch(e){
			CYBERSEARCH_Wm.CYBERSEARCH_log_message("Error creating CyberSearch export file.");
			return;
		}

		//Prep the file for writing
		try{
			var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
			stream.init(file, 0x02 | 0x08 | 0x20, 0666, 0);
		}
		catch(e){
			CYBERSEARCH_Wm.CYBERSEARCH_log_message("Error prepping CyberSearch export file.");
			return;
		}

		//Generate the content for the file
		var settings = {};
		settings.maxRichResults			= CYBERSEARCH_Wm.CYBERSEARCH_maxRichResults;
		settings.backgroundColor		= CYBERSEARCH_Wm.CYBERSEARCH_backgroundColor;
		settings.defaultEngine			= CYBERSEARCH_Wm.CYBERSEARCH_defaultEngine;
		settings.displayDescriptions	= CYBERSEARCH_Wm.CYBERSEARCH_displayDescriptions;
		settings.displayOptions			= CYBERSEARCH_Wm.CYBERSEARCH_displayOptions;
		settings.displayOptionsContext	= CYBERSEARCH_Wm.CYBERSEARCH_displayOptionsContext;
		settings.displaySnapback		= CYBERSEARCH_Wm.CYBERSEARCH_displaySnapback;
		settings.domainSearch			= CYBERSEARCH_Wm.CYBERSEARCH_domainSearch;
		settings.enterGoToGoogle		= CYBERSEARCH_Wm.CYBERSEARCH_enterGoToGoogle;
		settings.hideHistoryOnKeyword	= CYBERSEARCH_Wm.CYBERSEARCH_hideHistoryOnKeyword;
		settings.hideSearchNoKeyword	= CYBERSEARCH_Wm.CYBERSEARCH_hideSearchNoKeyword;
		settings.keywordPrefix			= CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefix;
		settings.keywordPrefixOnAll		= CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefixOnAll;
		settings.keywords				= CYBERSEARCH_Wm.CYBERSEARCH_keywords;
		settings.keywordsEnabled		= CYBERSEARCH_Wm.CYBERSEARCH_keywordsEnabled;
		settings.maxRows				= CYBERSEARCH_Wm.CYBERSEARCH_maxRows;
		settings.oneTimeVersion			= CYBERSEARCH_Wm.CYBERSEARCH_oneTimeVersion;
		settings.pageKeyword			= CYBERSEARCH_Wm.CYBERSEARCH_pageKeyword;
		settings.pageKeywordEnabled		= CYBERSEARCH_Wm.CYBERSEARCH_pageKeywordEnabled;

		settings.showThumbnails		= CYBERSEARCH_Wm.CYBERSEARCH_showThumbnails;

		var save_json = JSON.stringify(settings);

		//Write the content to the file
		try{
			stream.write(save_json, save_json.length);
			stream.close();
		}
		catch(e){
			CYBERSEARCH_Wm.CYBERSEARCH_log_message("Error writing CyberSearch export file.");
			return;
		}
	}
}


/*
Import CyberSearch settings from a text file
*/
function CYBERSEARCH_settings_import(){
	const nsIFilePicker = Components.interfaces.nsIFilePicker;
	var filepicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

	//Display the file picker for the user
	filepicker.init(window, "File Location", nsIFilePicker.modeOpen);
	filepicker.appendFilters(nsIFilePicker.filterText);
	filepicker.defaultExtension = "txt";
	var filepicker_code = filepicker.show();

	//Grab the path they chose
	if(filepicker_code == nsIFilePicker.returnOK){
		var import_path = filepicker.file.path;

		try{
			var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			var fstream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
			var cstream = Components.classes["@mozilla.org/intl/converter-input-stream;1"].createInstance(Components.interfaces.nsIConverterInputStream);
			var data = "";

			//Open file
			file.initWithPath(import_path);
			fstream.init(file, -1, 0, 0);
			cstream.init(fstream, "UTF-8", 0, 0);

			//Store the file's contents in "data" variable
			let (str = {}){
				cstream.readString(-1, str);
				data = str.value;
			}
			cstream.close();
		}
		catch(e){
			CYBERSEARCH_Wm.CYBERSEARCH_log_message("Error reading CyberSearch import file.");
			return;
		}

		//Parse the contents
		try{
			var settings = {};
			settings = JSON.parse(data);
		}
		catch(e){
			CYBERSEARCH_Wm.CYBERSEARCH_log_message("Error parsing CyberSearch import file.");
			return;
		}

		//Generate the content for the file
		CYBERSEARCH_Wm.CYBERSEARCH_maxRichResults			= settings.maxRichResults;
		CYBERSEARCH_Wm.CYBERSEARCH_backgroundColor			= settings.backgroundColor;
		CYBERSEARCH_Wm.CYBERSEARCH_displayDescriptions		= settings.displayDescriptions;
		CYBERSEARCH_Wm.CYBERSEARCH_displayOptions			= settings.displayOptions;
		CYBERSEARCH_Wm.CYBERSEARCH_displayOptionsContext	= settings.displayOptionsContext;
		CYBERSEARCH_Wm.CYBERSEARCH_displaySnapback			= settings.displaySnapback;
		CYBERSEARCH_Wm.CYBERSEARCH_domainSearch				= settings.domainSearch;
		CYBERSEARCH_Wm.CYBERSEARCH_enterGoToGoogle			= settings.enterGoToGoogle;
		CYBERSEARCH_Wm.CYBERSEARCH_hideHistoryOnKeyword		= settings.hideHistoryOnKeyword;
		CYBERSEARCH_Wm.CYBERSEARCH_hideSearchNoKeyword		= settings.hideSearchNoKeyword;
		CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefix			= settings.keywordPrefix;
		CYBERSEARCH_Wm.CYBERSEARCH_keywordPrefixOnAll		= settings.keywordPrefixOnAll;
		CYBERSEARCH_Wm.CYBERSEARCH_keywords					= settings.keywords;
		CYBERSEARCH_Wm.CYBERSEARCH_keywordsEnabled			= settings.keywordsEnabled;
		CYBERSEARCH_Wm.CYBERSEARCH_maxRows					= settings.maxRows;
		CYBERSEARCH_Wm.CYBERSEARCH_oneTimeVersion			= settings.oneTimeVersion;
		CYBERSEARCH_Wm.CYBERSEARCH_pageKeyword				= settings.pageKeyword;
		CYBERSEARCH_Wm.CYBERSEARCH_pageKeywordEnabled		= settings.pageKeywordEnabled;

		CYBERSEARCH_Wm.CYBERSEARCH_showThumbnails			= settings.showThumbnails;

		//Save the keywords
		var save_json = JSON.stringify(CYBERSEARCH_Wm.CYBERSEARCH_keywords);
		CYBERSEARCH_Wm.CYBERSEARCH_PREFS.setCharPref("extensions.cybersearch.keywords", save_json);

		//Update the interface with new options, then save them to the preferences
		CYBERSEARCH_options_initialize();
		CYBERSEARCH_options_save();

		alert("Your CyberSearch settings have successfully been imported.");
	}
}
