if (typeof CyberSearch != "object") { var CyberSearch = {}; }

const Cc				= Components.classes;
const Ci				= Components.interfaces;
const Cr				= Components.results;
const Cu				= Components.utils;
var Wm					= Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser");
var processed_results	= false;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");

function LOG(m) {
	m = 'CYBERSEARCH :: '+(new Date()).toGMTString()+"\n> "+m+"\n\n-- "+Components.stack.caller;
	dump(m + "\n");
	Cc['@mozilla.org/consoleservice;1'].getService(Ci.nsIConsoleService).logStringMessage(m);
}

CyberSearch.SearchAutoCompleteResult = function (result, search_result, maxNumResults){
	if(processed_results){return;}
	this._result		= result;
	this._search_result	= search_result;
	this._maxNumResults	= maxNumResults;
};

CyberSearch.SearchAutoCompleteResult.prototype = {
	get searchString(){
		try{
			return this._result.searchString;
		}
		catch(e){
			return "";
		}
	},

	get searchResult(){
		try{
			switch(this._result.searchResult){
				case Ci.nsIAutoCompleteResult.RESULT_NOMATCH:
					if(this._search_result.values == null)				return Ci.nsIAutoCompleteResult.RESULT_NOMATCH_ONGOING;
					else if(this._search_result.values.length > 0)		return Ci.nsIAutoCompleteResult.RESULT_SUCCESS;
					else if(this._search_result.values.length == 0)		return Ci.nsIAutoCompleteResult.RESULT_NOMATCH;
					break;
				case Ci.nsIAutoCompleteResult.RESULT_SUCCESS:
					if(this._search_result.values == null)				return Ci.nsIAutoCompleteResult.RESULT_SUCCESS_ONGOING;
					else if(this._search_result.values.length > 0)		return Ci.nsIAutoCompleteResult.RESULT_SUCCESS;
					else if(this._search_result.values.length == 0)		return Ci.nsIAutoCompleteResult.RESULT_SUCCESS;
					break;
				case Ci.nsIAutoCompleteResult.RESULT_NOMATCH_ONGOING:
					if(this._search_result.values == null)				return Ci.nsIAutoCompleteResult.RESULT_NOMATCH_ONGOING;
					else if(this._search_result.values.length > 0)		return Ci.nsIAutoCompleteResult.RESULT_SUCCESS_ONGOING;
					else if(this._search_result.values.length == 0)		return Ci.nsIAutoCompleteResult.RESULT_NOMATCH_ONGOING;
					break;
				case Ci.nsIAutoCompleteResult.RESULT_SUCCESS_ONGOING:
					if(this._search_result.values == null)				return Ci.nsIAutoCompleteResult.RESULT_SUCCESS_ONGOING;
					else if(this._search_result.values.length > 0)		return Ci.nsIAutoCompleteResult.RESULT_SUCCESS_ONGOING;
					else if(this._search_result.values.length == 0)		return Ci.nsIAutoCompleteResult.RESULT_SUCCESS_ONGOING;
					break;
				default:
					return this._result.searchResult;
			}
		}
		catch(e){}
	},

	get defaultIndex(){
		if(this._result.defaultIndex == -1 && this._search_result.values != null && this._search_result.values.length > 0){
			return 0;
		}
		return this._result.defaultIndex;
	},

	get errorDescription(){
		return this._result.errorDescription;
	},

	get matchCount(){
		try{
			var numResults = Math.min(this._result.matchCount, this._maxNumResults);
			if(this._search_result.values != null){
				numResults += this._search_result.values.length;
			}
			return numResults;
		}
		catch(e){
			return 0;
		}
	},

	getValueAt: function(index){//#1
		try{
			var numResults = Math.min(this._result.matchCount, this._maxNumResults);
			if(index < numResults){
				return this._result.getValueAt(index);
			}
			else{
				if(this._search_result.last[index - numResults]){
					processed_results = true;
				}
				return this._search_result.values[index - numResults];
			}
		}
		catch(e){
			return "";
		}
	},

	getLabelAt: function(index){//#1
		try{
			var numResults = Math.min(this._result.matchCount, this._maxNumResults);
			if(index < numResults){
				return this._result.getValueAt(index);
			}
			else{
				if(this._search_result.last[index - numResults]){
					processed_results = true;
				}
				return this._search_result.values[index - numResults];
			}
		}
		catch(e){
			return "";
		}
	},

	getCommentAt: function(index){//#3
		try{
			var numResults = Math.min(this._result.matchCount, this._maxNumResults);
			if(index < numResults){
				return this._result.getCommentAt(index);
			}
			else{
				return this._search_result.comments[index - numResults];
			}
		}
		catch(e){
			return "";
		}
	},

	getStyleAt: function(index){//#4
		try{
			var numResults = Math.min(this._result.matchCount, this._maxNumResults);
			if(index < numResults){
				return this._result.getStyleAt(index);
			}
			else{
				return "cybersearch";
			}
		}
		catch(e){
			return "cybersearch";
		}
	},

	getImageAt: function(index){//#2
	
		try{
			var numResults = Math.min(this._result.matchCount, this._maxNumResults);
			if(index < numResults){
				return this._result.getImageAt(index);
			}
			else{
				return this._search_result.images[index - numResults];
			}
		}
		catch(e){
			return "moz-anno:favicon:http://google.com/favicon.ico";
		}
	},

	removeValueAt: function(index, removeFromDb){
		try{
			var numResults = Math.min(this._result.matchCount, this._maxNumResults);
			if(index < numResults){
				return this._result.removeValueAt(index, removeFromDb);
			}
			else{
				this._search_result.values.splice(index - numResults, 1);
				this._search_result.comments.splice(index - numResults, 1);
				this._search_result.images.splice(index - numResults, 1);
			}
		}
		catch(e){
		}

	},

	QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIAutoCompleteResult]),
};




CyberSearch.GoogleSearch = function () {
	this.historyAutoComplete		= Components.classes["@mozilla.org/autocomplete/search;1?name=history"].getService(Components.interfaces.nsIAutoCompleteSearch);
}
CyberSearch.GoogleSearch.prototype =
{
	classDescription: "CyberSearch Google",
	classID:          Components.ID("5382f775-ac46-47e3-9807-a4c349e16be1"),
	//classID:          Components.ID("0569bc03-6a92-4786-bb5e-75a445907b4c"),
	contractID:       "@mozilla.org/autocomplete/search;1?name=cybersearch-google",
	QueryInterface:   XPCOMUtils.generateQI([Components.interfaces.nsIAutoCompleteSearch]),
	pendingRequest:   null,
	launchSearch:     function(cb) {
		if(this.pendingRequest)
			this.pendingRequest.cancel();
		let i = Ci.nsITimer, t = Cc["@mozilla.org/timer;1"].createInstance(i);
		t.initWithCallback({notify:cb},200,i.TYPE_ONE_SHOT);
		this.pendingRequest = t;
	},
	startSearch:      function(searchString, searchParam, previousResult, listener)
    {

		//Global window variable for accessing the browser window
		if(!Wm) {
			Wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser");
		}

		//Don't search unless Google is the engine being used
		if(Wm.CYBERSEARCH_processed.keyword_engine != "google"){
			return true;
		}
		this.stopSearch();

		//Reset flag
		processed_results = false;

		//Hide history results if a keyword is being used
		if(Wm.CYBERSEARCH_processed.keyword_used){
			var num_history_results = 0;
			var history_string		= Wm.CYBERSEARCH_RANDOM_STRING;
		}
		else{
			var num_history_results = Wm.CYBERSEARCH_maxRows;
			var history_string		= searchString;
		}

		//Search the user's history
		//this.historyAutoComplete		= Components.classes["@mozilla.org/autocomplete/search;1?name=history"].createInstance(Components.interfaces.nsIAutoCompleteSearch);
		var _search						= this;
		_search._result					= null;
		var internal_results			= {values: [], comments: [], images: [], last: []};
		autoCompleteObserver = {
			onSearchResult: function(search, result){
				_search._result = result;
				// listener.onSearchResult(_search, new CyberSearch.SearchAutoCompleteResult(result, internal_results, num_history_results));
			}
		};
		this.historyAutoComplete.startSearch(history_string, searchParam, null, autoCompleteObserver);

		//Don't search unless a keyword is recognized
		if(Wm.CYBERSEARCH_hideSearchNoKeyword && !Wm.CYBERSEARCH_processed.keyword_used){
			listener.onSearchResult(_search, new CyberSearch.SearchAutoCompleteResult(result, internal_results, num_history_results));
			return true;
		}

		var decodes = new Array(
			new Array('\\\\x22', '"'),
			new Array('\\\\x26', '&'),
			new Array('\\\\x27', '\''),
			new Array('\\\\x3c', '<'),
			new Array('\\\\x3e', '>'),
			new Array('\\\\x3d', '='),
			new Array('\\x22', '"'),
			new Array('\\x26', '&'),
			new Array('\\x27', '\''),
			new Array('\\x3c', '<'),
			new Array('\\x3e', '>'),
			new Array('\\x3d', '='),
			new Array('&amp;', '&'),
			new Array('\\\\', '')
		);
		var google_query = 'http://www.google.com/s?hl=en&sugexp=gsihc&xhr=t&q=' +  Wm.CYBERSEARCH_processed.engine_query + '&cp=' + Wm.CYBERSEARCH_processed.engine_query.length + '&pf=p&sclient=psy&site=webhp&aq=&aqi=&aql=&oq=&pbx=1&bav=on.2,or.r_gc.r_pw.&fp=' + Math.round(Math.random() * 1000)  + '&tch=1&ech=1&psi=' + new Date().getTime(); //Math.round(Math.random() * 1000);
		//Open the pipe
		var xhr	= Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance();
		xhr.open("GET", google_query, true);

		//Execute this when the query returns
		xhr.addEventListener("load", function(event){
			//Grab and parse the data
			var response = xhr.responseText;
			var description = '';
			var title = '';
			var url = '';
			for (var i=0; i<decodes.length; i++) {
				replace = RegExp( decodes[i][0], 'g' ) ;
				response= response.replace(replace, decodes[i][1]);
			}
			
			regexp = /<!--m.*?<!--n-->/gim;
			var response_matched = response.match(regexp);

			if(response_matched && response_matched.length < 2) {
				response = response.split("<li");
			} else {
				response = response_matched;
			}

			//Initialize the array of results
			Wm.CYBERSEARCH_results = new Array();

			if (response != null) {
				for(var i=0; i<response.length; i++) {
					//sub = response[i].substring(0, response[i].indexOf(">"));

					if(response[i].search(/class=(")?g/) == -1) {
						continue;
					}
					if(response[i].search(/http:\/\/maps.google/)  != -1) {
						continue;
					}

					if(response[i].search(/<li.*?id=/)  != -1) {
						if(!response[i].search(/<li.*?id=(")?mbb/)  == -1) {
							continue;
						}
					}

					/*
					description_start = response[i].indexOf('<span class=st');
					description = response[i].substring(description_start, (response[i].length-1));
					description_end = description.indexOf('</span>') + 7;
					description = description.substring(0, description_end);
					description = stripHTML(description);

					title_start = response[i].indexOf('<h3');
					title_end = response[i].indexOf('</h3>');
					title = response[i].substring(title_start, title_end);
					title = title.substring((title.indexOf('\')">') + 4), title.indexOf('</a>'));
					title = stripHTML(title);
					*/

					description = response[i].match(/<span class=.+?st.+?<\/span>/ig);

					if(description != null && description.length > 0)
						description = stripHTML(description[0]);


					title = response[i].match(/<h3.+?<\/h3>/ig);
					//title = title.substring((title.indexOf('\')">') + 4), title.indexOf('</a>'));
					
					if(title != null && title.length > 0)
						title = stripHTML(title[0]);

					var url = '';
					var urlsub = response[i].substring((response[i].search(/<a href="?/) + 9), (response[i].length-1));
					var pos = urlsub.search(/"? class=l/);
					var url = urlsub.substring(0, pos);


					//Initialize variables
					var result_info	= {};

					//If the user didn't specify an icon for the results just use the Google icon
					if(Wm.CYBERSEARCH_processed.keyword_icon){
						var result_icon = Wm.CYBERSEARCH_processed.keyword_icon;
					}
					else{
						var result_icon = "http://www.google.com/favicon.ico";
					}

					if (url == "") continue;
					
					//Format the results depending on what type of Google Search it is
					switch(Wm.CYBERSEARCH_processed.keyword_type){
						case "web":
							result_info.description		= description;
							result_info.icon			= "http://s2.googleusercontent.com/s2/favicons?domain=" + url.replace(new RegExp("^http(s)?:\\/\\/"),"").split("/")[0];
							// result_info.icon			= "http://" + url.replace(new RegExp("^http(s)?:\\/\\/"),"").split("/")[0] + "/favicon.ico";
							result_info.title			= title;
							result_info.titleNoFormat	= title;
							result_info.type			= "web";
							result_info.url				= url;
							break;
						default:
							result_info					= {};
					}

					//Replace HTML characters
					if(result_info.title){
						result_info.title = html_char_replace(result_info.title);
					}

					//Update the global variable
					Wm.CYBERSEARCH_results.push(result_info);

					//Push the result onto the queue
					internal_results.values.push(result_info.url);
					internal_results.comments.push(result_info.title);
					internal_results.images.push(result_info.icon);
					internal_results.last.push(false);
				}
			}

			//TODO: list URLs that are being filtered against in the message
			//Add the "Search Google" result
			//if(data != "error"){
			internal_results.values.push(Wm.CYBERSEARCH_processed.search_url);
			internal_results.comments.push("[Search Google" + Wm.CYBERSEARCH_processed.search_url_type + " for '" + Wm.CYBERSEARCH_processed.search_text + "']");
			internal_results.images.push("chrome://cybersearch/content/cybersearch_icon_16.png");
			internal_results.last.push(true);

			//Add the CyberSearch results
			if(_search._result != null)	listener.onSearchResult(_search, new CyberSearch.SearchAutoCompleteResult(_search._result, internal_results , 11));
			//}
			_search._req = null;
		}, false);

		//Send the query to Google
		this.launchSearch(function(){
			xhr.send(null);
			_search._req = xhr;
			_search.pendingRequest = null;
		});
	},

	stopSearch: function(){
		//Don't try to abort unless needed
		if((Wm.CYBERSEARCH_hideSearchNoKeyword && !Wm.CYBERSEARCH_processed.keyword_used) || Wm.CYBERSEARCH_processed.keyword_engine != "google"){
			return false;
		}

		this.historyAutoComplete.stopSearch();
		if(this._req){
		    this._req.abort();
			this._req = null;
		}
	}
};

/*
This is used to replace HTML characters in a string
*/
function html_char_replace(text){
	var html_replace_these	= new Array('&amp;','&quot;', '&#39;');
	var html_replace_with	= new Array('&','"', '\'');

	for(var i=0; i < html_replace_these.length; i++){
		text = text.replace(html_replace_these[i], html_replace_with[i], "g");
	}

	return text;
}

function stripHTML(str){
	// remove all string within tags
	var tmp = str.replace(/(<.*['"])([^'"]*)(['"]>)/g,
	function(x, p1, p2, p3) { return  p1 + p3;}
	);
	// now remove the tags
	return tmp.replace(/<\/?[^>]+>/gi, '');
}


function NSGetModule(compMgr, fileSpec)
{
    return XPCOMUtils.generateModule([CyberSearch.GoogleSearch]);
}

if(XPCOMUtils.generateNSGetFactory)
{
    var NSGetFactory = XPCOMUtils.generateNSGetFactory([CyberSearch.GoogleSearch]);
}